# -*- coding: utf-8 -*-
## $Id: interfaces.py 14141 2016-09-01 03:58:57Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

#from grok import getSite
from zope import schema
#from zope.interface import invariant, Invalid
#from zope.component import getUtility
#from zope.catalog.interfaces import ICatalog
from zc.sourcefactory.basic import BasicSourceFactory
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, academic_sessions_vocab, validate_email,
    IKofaObject, ContextualDictSourceFactoryBase)
from waeup.kofa.schema import FormattedDate, TextLineChoice, PhoneNumber
from waeup.kofa.students.vocabularies import nats_vocab, GenderSource
from waeup.kofa.refereeentries import RefereeEntryField
from waeup.kofa.applicants.interfaces import contextual_reg_num_source
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types,
    programme_types_vocab, jambsubjects,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit, INigeriaPGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    IBankAccount,
    )
from waeup.uniben.interfaces import MessageFactory as _
from waeup.uniben.payments.interfaces import ICustomOnlinePayment


REGISTRATION_CATS = {
    'corporate': ('Corporate Registration', 250000, 1),
    'group': ('Group Registration', 200000, 2),
    'individual': ('Individual Registration', 45000, 3),
    'student': ('Student Registration', 5000, 4),
    'fullpage': ('Full Page Advert', 250000, 5),
    'halfpage': ('Half Page Advert', 150000, 6),
    'quarterpage': ('Quarter Page Advert', 100000, 7),
    }

class RegTypesSource(BasicSourceFactory):
    """A source that delivers all kinds of registrations.
    """
    def getValues(self):
        sorted_items = sorted(REGISTRATION_CATS.items(),
                              key=lambda element: element[1][2])
        return [item[0] for item in sorted_items]

    def getTitle(self, value):
        return u"%s @ ₦ %s" % (
            REGISTRATION_CATS[value][0],
            REGISTRATION_CATS[value][1])

class IUnibenRegistration(IKofaObject):
    """A Uniben registrant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Registrant Id'),
        required = False,
        readonly = False,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    sex = schema.Choice(
        title = _(u'Sex'),
        source = GenderSource(),
        required = True,
        )

    nationality = schema.Choice(
        vocabulary = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    #perm_address = schema.Text(
    #    title = _(u'Current Local Address'),
    #    required = False,
    #    readonly = False,
    #    )

    institution = schema.TextLine(
        title = _(u'Institution/Organisation'),
        required = False,
        readonly = False,
        )

    city = schema.TextLine(
        title = _(u'City'),
        required = False,
        readonly = False,
        )

    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA'),
        required = False,
        )

    matric_number = schema.TextLine(
        title = _(u'Uniben Matriculation Number'),
        required = False,
        readonly = False,
        )

    registration_cats = schema.List(
        title = _(u'Registration Categories'),
        value_type = schema.Choice(source=RegTypesSource()),
        required = True,
        defaultFactory=list,
        )

#    @invariant
#    def matric_number_exists(applicant):
#        if applicant.matric_number:
#            catalog = getUtility(ICatalog, name='students_catalog')
#            accommodation_session = getSite()['hostels'].accommodation_session
#            student = catalog.searchResults(matric_number=(
#                applicant.matric_number, applicant.matric_number))
#            if len(student) != 1:
#                raise Invalid(_("Matriculation number not found."))

class ICustomUGApplicant(IApplicantBaseData, IBankAccount):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the UG_OMIT* tuples.
    """

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    #perm_address = schema.Text(
    #    title = _(u'Permanent Address'),
    #    required = False,
    #    )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )

    programme_type = schema.Choice(
        title = _(u'Programme Type'),
        vocabulary = programme_types_vocab,
        required = False,
        )

    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    fst_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )
    fst_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )
    fst_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    fst_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    fst_sit_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    scd_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )
    scd_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )
    scd_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    scd_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    scd_sit_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    jamb_subjects = schema.Text(
        title = _(u'Subjects and Scores'),
        required = False,
        )
    jamb_subjects_list = schema.List(
        title = _(u'JAMB Subjects'),
        required = False,
        defaultFactory=list,
        value_type = schema.Choice(
            vocabulary = jambsubjects
            #source = JAMBSubjectSource(),
            ),
        )
    jamb_score = schema.Int(
        title = _(u'Total JAMB Score'),
        required = False,
        )
    #jamb_age = schema.Int(
    #    title = _(u'Age (provided by JAMB)'),
    #    required = False,
    #    )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = False,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Int(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    aggregate = schema.Int(
        title = _(u'Aggregate Score (%)'),
        description = _(u'(average of relative JAMB and PUTME scores)'),
        required = False,
        )
    result_uploaded = schema.Bool(
        title = _(u'Result uploaded'),
        default = False,
        required = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

ICustomUGApplicant[
    'locked'].order =  IApplicantBaseData['suspended'].order
ICustomUGApplicant[
    'result_uploaded'].order =  ICustomUGApplicant['suspended'].order

class ICustomPGApplicant(INigeriaPGApplicant):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """

    referees = schema.List(
        title = _(u'Referees'),
        value_type = RefereeEntryField(),
        required = False,
        defaultFactory=list,
        )

ICustomPGApplicant[
    'referees'].order =  INigeriaPGApplicant['emp2_reason'].order

class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant,
    IUnibenRegistration):
    """An interface for both types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicant data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(INigeriaUGApplicantEdit):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomPGApplicantEdit(INigeriaPGApplicantEdit):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    referees = schema.List(
        title = _(u'Referees'),
        value_type = RefereeEntryField(),
        required = False,
        defaultFactory=list,
        )

ICustomPGApplicantEdit[
    'referees'].order =  INigeriaPGApplicantEdit['emp2_reason'].order

class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class IPUTMEApplicantEdit(IPUTMEApplicantEdit):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantUpdateByRegNo(INigeriaApplicantUpdateByRegNo):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """
